import { resolve, join } from 'node:path';
import { existsSync, mkdirSync, readFileSync, writeFileSync } from 'node:fs';
import { buildSync } from 'esbuild';
import { languageWorksByLabel } from './lnaguageWork.js';
import { workerMiddleware, cacheDir, getFilenameByEntry, getWorkPath } from './workerMiddleware.js';
/**
 * Return a resolved path for a given Monaco file.
 */
export function resolveMonacoPath(filePath) {
    try {
        return resolve(join(process.cwd(), 'node_modules', filePath));
    }
    catch (err) {
        return resolve(filePath);
    }
}
export function getWorks(options) {
    let works = options.languageWorkers.map((work) => languageWorksByLabel[work]);
    works.push(...options.customWorkers);
    return works;
}
export default function monacoEditorPlugin(options) {
    const languageWorkers = options.languageWorkers || Object.keys(languageWorksByLabel);
    const publicPath = options.publicPath || 'monacoeditorwork';
    const globalAPI = options.globalAPI || false;
    const customWorkers = options.customWorkers || [];
    const forceBuildCDN = options.forceBuildCDN || false;
    options = Object.assign(Object.assign({}, options), { languageWorkers,
        publicPath,
        globalAPI,
        customWorkers,
        forceBuildCDN });
    let resolvedConfig;
    return {
        name: 'vite-plugin-monaco-editor-new',
        configResolved(getResolvedConfig) {
            resolvedConfig = getResolvedConfig;
        },
        configureServer(server) {
            if (isCDN(publicPath)) {
                return;
            }
            workerMiddleware(server.middlewares, resolvedConfig, options);
        },
        transformIndexHtml(html) {
            const works = getWorks(options);
            const workerPaths = getWorkPath(works, options, resolvedConfig);
            const globals = {
                MonacoEnvironment: `(function (paths) {
          return {
            globalAPI: ${globalAPI},
            getWorkerUrl : function (moduleId, label) {
              var result =  paths[label];
              if (/^((http:)|(https:)|(file:)|(\\/\\/))/.test(result)) {
                var currentUrl = String(window.location);
                var currentOrigin = currentUrl.substr(0, currentUrl.length - window.location.hash.length - window.location.search.length - window.location.pathname.length);
                if (result.substring(0, currentOrigin.length) !== currentOrigin) {
                  var js = '/*' + label + '*/importScripts("' + result + '");';
                  var blob = new Blob([js], { type: 'application/javascript' });
                  return URL.createObjectURL(blob);
                }
              }
              return result;
            }
          };
        })(${JSON.stringify(workerPaths, null, 2)})`,
            };
            const descriptor = [
                {
                    tag: 'script',
                    children: Object.keys(globals)
                        .map((key) => `self[${JSON.stringify(key)}] = ${globals[key]};`)
                        .join('\n'),
                    injectTo: 'head-prepend',
                },
            ];
            return descriptor;
        },
        writeBundle() {
            // 是cdn地址并且没有强制构建worker cdn则返回
            if (isCDN(publicPath) && !forceBuildCDN) {
                return;
            }
            const works = getWorks(options);
            const distPath = options.customDistPath
                ? options.customDistPath(resolvedConfig.root, resolvedConfig.build.outDir, resolvedConfig.base)
                : join(resolvedConfig.root, resolvedConfig.build.outDir, resolvedConfig.base, options.publicPath);
            //  console.log("distPath", distPath)
            // write publicPath
            if (!existsSync(distPath)) {
                mkdirSync(distPath, {
                    recursive: true,
                });
            }
            for (const work of works) {
                if (!existsSync(cacheDir + getFilenameByEntry(work.entry))) {
                    buildSync({
                        entryPoints: [resolveMonacoPath(work.entry)],
                        bundle: true,
                        outfile: cacheDir + getFilenameByEntry(work.entry),
                    });
                }
                const contentBuffer = readFileSync(cacheDir + getFilenameByEntry(work.entry));
                const workDistPath = resolve(distPath, getFilenameByEntry(work.entry));
                writeFileSync(workDistPath, contentBuffer);
            }
        },
    };
}
export function isCDN(publicPath) {
    if (/^((http:)|(https:)|(file:)|(\/\/))/.test(publicPath)) {
        return true;
    }
    return false;
}
